@@ -426,6 +426,9 @@ RegressTestBoxed *regress_test_boxed_copy     (RegressTestBoxed *boxed);
 gboolean   regress_test_boxed_equals   (RegressTestBoxed *boxed,
 					RegressTestBoxed *other);
 
+void       regress_test_boxeds_not_a_method (RegressTestBoxed *boxed);
+void       regress_test_boxeds_not_a_static (void);
+
 typedef struct _RegressTestBoxedB RegressTestBoxedB;
 
 struct _RegressTestBoxedB
@@ -485,8 +488,13 @@ struct _RegressTestObj
   double some_double;
   char* string;
   GType gtype;
+
+  /* < private > */
+  void (*function_ptr) (void);
 };
 
+typedef void (*RegressTestExternallyDefinedCallback) (RegressTestObj *obj, int someint);
+
 struct _RegressTestObjClass
 {
   GObjectClass parent_class;
@@ -502,6 +510,8 @@ struct _RegressTestObjClass
   guint test_signal;
   guint test_signal_with_static_scope_arg;
 
+  RegressTestExternallyDefinedCallback complex_vfunc;
+
   /* Should be replaced with simple "gpointer" and not be callback */
   void (*_regress_reserved1) (void);
   void (*_regress_reserved2) (void);
